%
% Simple numerical solution for the Buckley-Leverett equation given by

% dS/dt + df/dx = 0, where

% S is the saturation
% f is the fractional flow function

% The flux function used is f = S^2/(S^2 + 0.5*(1-S)^2)

% We simulate the equation for x in [0,1]
% 
% We use a simple explicit time stepping method with a straightforward
% upwind discretization (note here that flow is always to the right)
%
% The discretization gives
% Snew - Sold + dt/dx (f - fupwind) = 0

% We also plot the fractional flow function and its derivative
%


% Set problem parameters

dx = 0.01;

% Choose the time step so that the stability condition is met
% Note that the largest speed is 4
dt = 0.1*dx;
%dt = 0.8*dx;

% Set total simulation time
tfinal = 1;

xarray = -0.5:dx:1.5;
n=length(xarray)-1;

Sleft = 1;
Sright = 0;

S = Sright*ones(n+1,1);
S(1:n/4+1)= Sleft*ones(n/4+1,1);
Snew = S;

coef = dt/dx;

% Plot the fractional flow function and its derivative as a function of 
% saturation varying between 0 and 1

Sf =[0:0.01:1]'; nSf = 100;
fracflow = (Sf.^2) ./ (Sf.^2 + 0.5*( ones(nSf+1,1) - Sf ).^2 );
fracflowder = diff(fracflow)/dx;

subplot(1,2,1); plot(Sf, fracflow);
title('fractional flow function');

subplot(1,2,2); plot(Sf(1:nSf), fracflowder);
title('derivative of fractional flow function');

t=0;


% plot solutions in a new figure
figure;

fracflow = (S.^2) ./ (S.^2 + 0.5*( ones(n+1,1) - S ).^2 );

nt=0;
while t<tfinal,
    
    Snew(2:n+1) = S(2:n+1) - coef*diff(fracflow);
    
    Snew(1) = Sleft;
    S = Snew;
   
% Make the following comments actual code if you want to see the
% Solution at teach time step.
% Just hit "return" to move to the next time step

    if (rem(nt,10)==0) 
        plot(xarray,S,'+');
        exactBL;
        pause;
    end
    
    t=t+dt; nt=nt+1;
    fracflow = (S.^2) ./ (S.^2 + 0.5*( ones(n+1,1) - S ).^2);
    
end;

figure;
plot(xarray,S);
exactBL;
title('Solution at time t=0.5');

